# ==================================================================
# Plugins::Dbsql::DateSearch - Auto Generated Program Module
#
#   Plugins::Dbsql::DateSearch
#   Author  : Jasper Cramwinckel
#   Version : 0.2
#   Updated : Tue Nov  5 13:45:46 2002
#
# ==================================================================
#

package Plugins::Dbsql::DateSearch;
# ==================================================================

use strict;
use Data::Dumper;
use Date::Calc qw(Monday_of_Week Add_Delta_Days Days_in_Month Add_Delta_YM ); 
use vars qw/$DEBUG/;
use GT::Base;
use GT::Plugins qw/STOP CONTINUE/;
use Dbsql qw/$IN $DB $CFG/;

# Inherit from base class for debug and error methods
@Plugins::Dbsql::DateSearch::ISA = qw(GT::Base);

# Your code begins here! Good Luck!

#post: return '' if $value is outside the range
sub CheckMinMax {
    my( $value, $min, $max ) = @_;
    defined($value) or return '';
	$value =~ /^(\-?\d+)$/ or return '';
    $1 <= $max or return '';
    $1 >= $min or return '';
    return $1;
}

sub CheckYear     { return CheckMinMax( @_, 1970, 2100 ); }
sub CheckMonth    { return CheckMinMax( @_, 1,    12   ); }
sub CheckMonthday { return CheckMinMax( @_, 1,    31   ); }
sub CheckYearday  { return CheckMinMax( @_, 1,    366  ); }
sub CheckWeek     { return CheckMinMax( @_, 0,    53   ); }

sub CompareDates {
    my( $y1, $m1, $d1, $y2, $m2, $d2, $rounddown ) = @_;
    ($y1 && $m1 && $d1) or return ( $y2, $m2, $d2 );
    ($y2 && $m2 && $d2) or return ( $y1, $m1, $d1 );
    $y1 != $y2 and return( ($rounddown xor $y1>$y2) ? ( $y2, $m2, $d2 ) : ( $y1, $m1, $d1 ) );
    $m1 != $m2 and return( ($rounddown xor $m1>$m2) ? ( $y2, $m2, $d2 ) : ( $y1, $m1, $d1 ) );
                   return( ($rounddown xor $d1>$d2) ? ( $y1, $m1, $d1 ) : ( $y2, $m2, $d2 ) );
}

sub GetDate {
    my( $column, $rounddown ) = @_;
        $DEBUG .= "<br>start GetDate($column, $rounddown )";

    #get the variables
    my $year     = CheckYear(    $IN->param($column."year"));
    my $month    = CheckMonth(   $IN->param($column."month"));
    my $monthday = CheckMonthday($IN->param($column."monthday"));
    my $yearday  = CheckYearday( $IN->param($column."yearday"));
    my $week     = CheckWeek(    $IN->param($column."week"));

    my $tmp = $IN->param($column."year_month") || '';
        $DEBUG .= "<BR>$column"."year_month -> '$tmp'";
    $tmp =~ /^(\d{4})-(\d{1,2})$/;
    $tmp = CheckYear($1) and (!$year or $tmp > $year) and $year = $tmp;
    $tmp = CheckMonth($2) and (!$month or $tmp > $month) and $month = $tmp;

    $tmp = $IN->param($column."year_week") || '';
    $tmp =~ /^(\d{4})-(\d{1,2})$/;
    $tmp = CheckYear($1) and (!$year or $tmp > $year) and $year = $tmp;
    $tmp = CheckWeek($2) and (!$week or $tmp > $week) and $week = $tmp;

    $tmp = $IN->param($column."year_month_monthday") || '';
    $tmp =~ /^(\d{4})-(\d{1,2})-(\d{1,2})$/;
    $tmp = CheckYear($1) and (!$year or $tmp > $year) and $year = $tmp;
    $tmp = CheckMonth($2) and (!$month or $tmp > $month) and $month = $tmp;
    $tmp = CheckMonthday($3) and (!$monthday or $tmp > $monthday) and $monthday = $tmp;

    $tmp = $IN->param($column."year_yearday") || '';
    $tmp =~ /^(\d{4})-(\d{1,3})$/;
    $tmp = CheckYear($1) and (!$year or $tmp > $year) and $year = $tmp;
    $tmp = CheckYearday($2) and (!$yearday or $tmp > $yearday) and $yearday = $tmp;

    #create dates from the variables and select one
    my @date = ();
    my @tmpdate = ();

    if( $year ) {
		@date = $rounddown ? ($year,1,1) : ($year,12,31);
	} else {
		return ();
	}

    if( $week ) {
        @tmpdate = Add_Delta_Days( Monday_of_Week( $week, $year ), ( $rounddown ? 0 : 6 ) ); 
        #check that $days is not in the next year
        $tmpdate[0] > $year and @tmpdate = ($year, 12, 31 );
        @date = CompareDates( @date, @tmpdate, $rounddown );
	}

    if( $month ) {
        @tmpdate = ( $year, $month, ($monthday?$monthday:($rounddown?1:Days_in_Month($year,$month))));
        @date = CompareDates( @date, @tmpdate, $rounddown );
    }

    if( $yearday ) {
        @tmpdate = Add_Delta_Days( $year, 1, 1, $yearday - 1 );
        @date = CompareDates( @date, @tmpdate, $rounddown );
    }

    my $daysfromnow = CheckMinMax($IN->param($column."daysfromnow"), -100000, 100000 );
    if($daysfromnow ne '') {
        @tmpdate = Add_Delta_Days( Today(), $daysfromnow );
        @date = CompareDates( @date, @tmpdate, $rounddown );
	}

    return @date;
}


# PLUGIN HOOKS
# ===================================================================


sub ProcessVariables {
# -------------------------------------------------------------------
# This subroutine will get called whenever the hook 'search_results'
# is run. You should call GT::Plugins->action ( STOP ) if you don't 
# want the regular code to run, otherwise the code will continue as
# normal.
#
    my (@args) = @_;
    my $home = $args[0];

    my %columns = ();
    foreach ( $IN->param() ) {
		/^(.*)-datesearch$/ and $columns{$1}++;
	}
    foreach my $column ( keys %columns ) {
        $DEBUG .= "<br>working with column $column";
        my @begindate = GetDate( "$column-first_", 1 );
        my @enddate = GetDate( "$column-last_", 0 );
        $DEBUG .= "<br>begindate:".join('-', @begindate);

        if( @begindate ) {
			my @tmpdate = ();        
			my $tmp = CheckMinMax( $IN->param( "$column-interval_years" ), -30, 30 );
			($tmp ne '') and @tmpdate = Add_Delta_YM( @begindate, $tmp, 0 );
			@enddate = CompareDates( @enddate, @tmpdate, 0 );
			
			$tmp = CheckMinMax( $IN->param( "$column-interval_months" ), -10000, 10000 );
			($tmp ne '') and @tmpdate = Add_Delta_YM( @begindate, 0, $tmp );
			@enddate = CompareDates( @enddate, @tmpdate, 0 );
			
			$tmp = CheckMinMax( $IN->param( "$column-interval_weeks" ), -10000, 10000 );
			($tmp ne '') and @tmpdate = Add_Delta_Days( @begindate, 7 * $tmp );
			@enddate = CompareDates( @enddate, @tmpdate, 0 );
			
			$tmp = CheckMinMax( $IN->param( "$column-interval_days" ), -10000, 10000 );
			($tmp ne '') and @tmpdate = Add_Delta_Days( @begindate, $tmp );
			@enddate = CompareDates( @enddate, @tmpdate, 0 );
		}

		if( @enddate and $IN->param( "$column-last_date_exclusive" ) ) {
			@enddate = Add_Delta_Days( @enddate, -1 );
		}

		my $columnname = $IN->param( "$column-columnname" ) || $column;
        @begindate and $home->{cgi}->{"$columnname-ge"} = join('-', @begindate);
        @enddate and $home->{cgi}->{"$columnname-le"} = join('-', @enddate);
    }

    $home->{cgi}->{datesearch_debug} = $DEBUG;
    return @args;
}

# ADMIN MENU OPTIONS
# ===================================================================

sub help {
# -------------------------------------------------------------------
# This subroutine will get called whenever the user clicks
# on 'Help' in the admin menu. Remember, you need to print
# your own content-type headers; you should use 
#
#   print $IN->header();
#
    print $IN->header();
    print qq~
<html>
    <head><title>datesearch plugin</title>
    <STYLE TYPE="text/css">
	td {vertical-align:top; font-family:Tahoma,Arial,Helvetica;font-size: smaller }
</STYLE>
    </head>
    <body bgcolor="#FFFFFF">
    ~;
    print Dbsql::header('datesearch Plugin', qq~
We are logging again!
The datesearch plugin will modify <code><i>datecolumnname</i>-le</code> and 
<code><i>datecolumnname</i>-ge</code> for all specified datecolumns. It
will also set the tag <code>datesearch_debug</code> with debug info.
<p />Valid parameters:
<table border="1">
  <tr>
    <td><code><i>datecolumnname</i>-datesearch</code></td>
    <td>Must be set to true to enable the plugin for the specified column</td>
  </tr>
  <tr><td height="1" colspan="2"><hr /></td></tr>
  <tr>
    <td><code><i>datecolumnname</i>-columnname</code></td>
    <td></td>
  </tr>
  <tr>
    <td><code><i>datecolumnname</i>-last_date_exclusive</code></td>
    <td>Whether the specified last date should be included or not. Defaults
        to false.</td>
  </tr>
  <tr><td height="1" colspan="2"><hr /></td></tr>
  <tr>
    <td><code><i>datecolumnname</i>-first_year</code></td>
    <td></td>
  </tr>
  <tr>
    <td><code><i>datecolumnname</i>-first_month</code></td>
    <td></td>
  </tr>
  <tr>
    <td><code><i>datecolumnname</i>-first_monthday</code></td>
    <td></td>
  </tr>
  <tr>
    <td><code><i>datecolumnname</i>-first_yearday</code></td>
    <td></td>
  </tr>
  <tr>
    <td><code><i>datecolumnname</i>-first_week</code></td>
    <td></td>
  </tr>
  <tr>
    <td><code><i>datecolumnname</i>-first_year_month</code></td>
    <td></td>
  </tr>
  <tr>
    <td><code><i>datecolumnname</i>-first_year_month_monthday</code></td>
    <td></td>
  </tr>
  <tr>
    <td><code><i>datecolumnname</i>-first_year_week</code></td>
    <td></td>
  </tr>
  <tr>
    <td><code><i>datecolumnname</i>-first_year_yearday</code></td>
    <td></td>
  </tr>
  <tr>
    <td><code><i>datecolumnname</i>-first_daysfromnow</code></td>
    <td></td>
  </tr>
  <tr><td height="1" colspan="2"><hr /></td></tr>
  <tr>
    <td><code><i>datecolumnname</i>-last_year</code></td>
    <td></td>
  </tr>
  <tr>
    <td><code><i>datecolumnname</i>-last_month</code></td>
    <td></td>
  </tr>
  <tr>
    <td><code><i>datecolumnname</i>-last_monthday</code></td>
    <td></td>
  </tr>
  <tr>
    <td><code><i>datecolumnname</i>-last_yearday</code></td>
    <td></td>
  </tr>
  <tr>
    <td><code><i>datecolumnname</i>-last_week</code></td>
    <td></td>
  </tr>
  <tr>
    <td><code><i>datecolumnname</i>-last_year_month</code></td>
    <td></td>
  </tr>
  <tr>
    <td><code><i>datecolumnname</i>-last_year_month_monthday</code></td>
    <td></td>
  </tr>
  <tr>
    <td><code><i>datecolumnname</i>-last_year_week</code></td>
    <td></td>
  </tr>
  <tr>
    <td><code><i>datecolumnname</i>-last_year_yearday</code></td>
    <td></td>
  </tr>
  <tr>
    <td><code><i>datecolumnname</i>-last_daysfromnow</code></td>
    <td></td>
  </tr>
  <tr><td height="1" colspan="2"><hr /></td></tr>
  <tr>
    <td><code><i>datecolumnname</i>-interval_years</code></td>
    <td></td>
  </tr>
  <tr>
    <td><code><i>datecolumnname</i>-interval_months</code></td>
    <td></td>
  </tr>
  <tr>
    <td><code><i>datecolumnname</i>-interval_weeks</code></td>
    <td></td>
  </tr>
 <tr>
    <td><code><i>datecolumnname</i>-interval_days</code></td>
    <td></td>
  </tr>
</table>
Todo:
<ul>
  <li>Add explenation to help text</li>
  <li>Add <code>interval_type</code> and <code>interval_value</code></li>
  <li>Test all options</li>
  <li>Add time options</li>
</ul>
        ~);
    print qq~
    </body>
</html>
    ~;

}

# Always end with a 1.
1;
